@@ -49,31 +49,38 @@ static uint8_t dhcp_end[] = {
 
 /* Both iovecs below have to have the same structure, since dhcp_send()
    pokes at the internals */
-#define DHCP_IOV_LEN 6
+#define DHCP_IOV_LEN 7
 
-static struct iovec dhcp_discover_iov[] = {
+static struct iovec dhcp_discover_iov[DHCP_IOV_LEN] = {
 	/* [0] = ip + udp header */
 	/* [1] = bootp header */
 	[2] = {dhcp_discover_hdr, sizeof(dhcp_discover_hdr)},
 	[3] = {dhcp_params, sizeof(dhcp_params)},
-	/* [4] = DHCP vendor class */
-	[5] = {dhcp_end, sizeof(dhcp_end)}
+	/* [4] = optional vendor class */
+	/* [5] = optional hostname */
+	/* [6] = {dhcp_end, sizeof(dhcp_end)} */
 };
 
-static struct iovec dhcp_request_iov[] = {
+static struct iovec dhcp_request_iov[DHCP_IOV_LEN] = {
 	/* [0] = ip + udp header */
 	/* [1] = bootp header */
 	[2] = {dhcp_request_hdr, sizeof(dhcp_request_hdr)},
 	[3] = {dhcp_params, sizeof(dhcp_params)},
-	/* [4] = DHCP vendor class */
-	[5] = {dhcp_end, sizeof(dhcp_end)}
+	/* [4] = optional vendor class */
+	/* [5] = optional hostname */
+	/* [6] = {dhcp_end, sizeof(dhcp_end)} */
 };
 
 /*
  * Parse a DHCP response packet
+ * Returns:
+ * 0 = Not handled
+ * 2 = DHCPOFFER (from dhcp_proto.h)
+ * 5 = DHCPACK
+ * 6 = DHCPNACK
  */
-static int
-dhcp_parse(struct netdev *dev, struct bootp_hdr *hdr, uint8_t * exts, int extlen)
+static int dhcp_parse(struct netdev *dev, struct bootp_hdr *hdr,
+		      uint8_t * exts, int extlen)
 {
 	uint8_t type = 0;
 	uint32_t serverid = INADDR_NONE;
@@ -101,19 +108,19 @@ dhcp_parse(struct netdev *dev, struct bootp_hdr *hdr, uint8_t * exts, int extlen
 
 	switch (type) {
 	case DHCPOFFER:
-		ret = bootp_parse(dev, hdr, exts, extlen);
-		if (ret == 1 && serverid != INADDR_NONE)
+		ret = bootp_parse(dev, hdr, exts, extlen) ? DHCPOFFER : 0;
+		if (ret == DHCPOFFER && serverid != INADDR_NONE)
 			dev->serverid = serverid;
 		DEBUG(("\n   dhcp offer\n"));
 		break;
 
 	case DHCPACK:
-		ret = bootp_parse(dev, hdr, exts, extlen);
+		ret = bootp_parse(dev, hdr, exts, extlen) ? DHCPACK : 0;
 		DEBUG(("\n   dhcp ack\n"));
 		break;
 
 	case DHCPNAK:
-		ret = 2;
+		ret = DHCPNAK;
 		DEBUG(("\n   dhcp nak\n"));
 		break;
 	}
@@ -122,6 +129,12 @@ dhcp_parse(struct netdev *dev, struct bootp_hdr *hdr, uint8_t * exts, int extlen
 
 /*
  * Receive and parse a DHCP packet
+ * Returns:
+ *-1 = Error in packet_recv
+ * 0 = Not handled
+ * 2 = DHCPOFFER (from dhcp_proto.h)
+ * 5 = DHCPACK
+ * 6 = DHCPNACK
  */
 static int dhcp_recv(struct netdev *dev)
 {
@@ -140,8 +153,8 @@ static int dhcp_recv(struct netdev *dev)
 
 	DEBUG(("\n   dhcp xid %08x ", dev->bootp.xid));
 
-	if (ret < sizeof(struct bootp_hdr) || bootp.op != BOOTP_REPLY ||	/* RFC951 7.5 */
-	    bootp.xid != dev->bootp.xid ||
+	if (ret < sizeof(struct bootp_hdr) || bootp.op != BOOTP_REPLY ||
+	    /* RFC951 7.5 */ bootp.xid != dev->bootp.xid ||
 	    memcmp(bootp.chaddr, dev->hwaddr, 16))
 		return 0;
 
@@ -153,6 +166,8 @@ static int dhcp_recv(struct netdev *dev)
 static int dhcp_send(struct netdev *dev, struct iovec *vec)
 {
 	struct bootp_hdr bootp;
+	char dhcp_hostname[SYS_NMLN+2];
+	int i = 4;
 
 	memset(&bootp, 0, sizeof(struct bootp_hdr));
 
@@ -160,7 +175,8 @@ static int dhcp_send(struct netdev *dev, struct iovec *vec)
 	bootp.htype	= dev->hwtype;
 	bootp.hlen	= dev->hwlen;
 	bootp.xid	= dev->bootp.xid;
-	bootp.ciaddr	= dev->ip_addr;
+	bootp.ciaddr	= INADDR_NONE;
+	bootp.yiaddr	= dev->ip_addr;
 	bootp.giaddr	= INADDR_ANY;
 	bootp.secs	= htons(time(NULL) - dev->open_time);
 	memcpy(bootp.chaddr, dev->hwaddr, 16);
@@ -168,12 +184,35 @@ static int dhcp_send(struct netdev *dev, struct iovec *vec)
 	vec[1].iov_base	= &bootp;
 	vec[1].iov_len	= sizeof(struct bootp_hdr);
 
-	vec[4].iov_base = vendor_class_identifier;
-	vec[4].iov_len  = vendor_class_identifier_len;
-
 	DEBUG(("xid %08x secs %d ", bootp.xid, ntohs(bootp.secs)));
 
-	return packet_send(dev, vec, DHCP_IOV_LEN);
+	if (vendor_class_identifier_len > 2) {
+		vec[i].iov_base = vendor_class_identifier;
+		vec[i].iov_len  = vendor_class_identifier_len;
+		i++;
+
+		DEBUG(("vendor_class_identifier \"%.*s\" ", 
+		       vendor_class_identifier_len-2, 
+		       vendor_class_identifier+2));
+	}
+
+	if (dev->reqhostname[0] != '\0') {
+		int len = strlen(dev->reqhostname);
+		dhcp_hostname[0] = 12;
+		dhcp_hostname[1] = len;
+		memcpy(dhcp_hostname+2, dev->reqhostname, len);
+
+		vec[i].iov_base = dhcp_hostname;
+		vec[i].iov_len  = len+2;
+		i++;
+
+		DEBUG(("hostname %.*s ", len, dhcp_hostname+2));
+	}
+
+	vec[i].iov_base = dhcp_end;
+	vec[i].iov_len  = sizeof(dhcp_end);
+
+	return packet_send(dev, vec, i);
 }
 
 /*
