@@ -1,6 +1,5 @@
-/*
- * nfsmount/nfsmount.h
- */
+#ifndef NFSMOUNT_NFSMOUNT_H
+#define NFSMOUNT_NFSMOUNT_H
 
 #include <linux/nfs_mount.h>
 
@@ -8,7 +7,7 @@ extern int nfs_port;
 
 extern int nfsmount_main(int argc, char *argv[]);
 int nfs_mount(const char *rem_name, const char *hostname,
-	      __u32 server, const char *rem_path,
+	      uint32_t server, const char *rem_path,
 	      const char *path, struct nfs_mount_data *data);
 
 enum nfs_proto {
@@ -32,3 +31,12 @@ enum nfs_proto {
 #else
 #define DEBUG(x) do { } while(0)
 #endif
+
+#ifndef MNTPROC_MNT
+#define MNTPROC_MNT 1
+#endif
+#ifndef MNTPROC_UMNT
+#define MNTPROC_UMNT 3
+#endif
+
+#endif /* NFSMOUNT_NFSMOUNT_H */
